@@ -7,10 +7,10 @@
 use std::sync::{Arc, LazyLock};
 
 use axum::{
-    extract::{Form, Query, State},
+    extract::{Form, State},
     response::{Html, IntoResponse, Response},
 };
-use axum_extra::typed_header::TypedHeader;
+use axum_extra::{extract::Query, typed_header::TypedHeader};
 use hyper::StatusCode;
 use mas_axum_utils::{
     InternalError, SessionInfoExt,
