@@ -9,6 +9,7 @@ use chrono::{DateTime, Utc};
 use mas_data_model::{Clock, UpstreamOAuthProvider, UpstreamOAuthProviderClaimsImports};
 use mas_storage::{
     Page, Pagination,
+    pagination::Node,
     upstream_oauth2::{
         UpstreamOAuthProviderFilter, UpstreamOAuthProviderParams, UpstreamOAuthProviderRepository,
     },
@@ -74,6 +75,12 @@ struct ProviderLookup {
     on_backchannel_logout: String,
 }
 
+impl Node<Ulid> for ProviderLookup {
+    fn cursor(&self) -> Ulid {
+        self.upstream_oauth_provider_id.into()
+    }
+}
+
 impl TryFrom<ProviderLookup> for UpstreamOAuthProvider {
     type Error = DatabaseInconsistencyError;
 
